<?php


namespace Fastrack\Location;


use Ballen\Distical\Calculator;
use Ballen\Distical\Entities\LatLong;
use Fastrack\Gallery\Gallery;
use Fastrack\Property\FeatureGroup;
use Fastrack\Property\Property;
use Fastrack\Utilities\Slug;
use Psr\Log\LoggerInterface;
use SilverStripe\Core\Config\Config;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;

/**
 * @method ManyManyList Properties
 */
class Place extends DataObject implements PermissionProvider
{

	private static $table_name = 'place';

	private static array $db = [
		'Name' => 'Varchar',
		'Slug' => 'Varchar',
	];

	private static array $has_one = [
		'Category' => PlaceCategory::class,
	];

	private static array $many_many = [
		'Properties' => Property::class,
	];

	private static array $translate = [
		'Name',
	];

	private static $indexes = [
		'SlugIndex' => [
			'type' => 'unique',
			'columns' => ['Slug'],
		],
	];

	private static bool $auto_relate_places = true;

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		$fields->removeByName('Properties');
		return $fields;
	}

	public function onBeforeWrite() {
		$this->Slug = Slug::generateSlug($this->Name, $this->Slug);
		$this->Title = $this->Name;
		parent::onBeforeWrite();
	}

	protected function onAfterWrite()
	{
		$this->addPropertiesWithinRadius();
		parent::onAfterWrite();
	}

	private function addPropertiesWithinRadius()
	{
		if (!Config::inst()->get(Place::class, 'auto_relate_places')) {
			return;
		}
		if ($this->Latitude && $this->Longitude) {
			$placeCoordinates = new LatLong($this->Latitude, $this->Longitude);
			$place = Place::get()->byID($this->ID);
			$properties = Property::get();
			foreach ($properties as $property) {
				$place->Properties()->remove($property);
				if ($property->Latitude && $property->Longitude) {
					try {
						$propertyCoordinates = new LatLong($property->Latitude, $property->Longitude);
						$distance = new Calculator($placeCoordinates, $propertyCoordinates);
						$radius = $property->PlaceRadius ? $property->PlaceRadius : 50;
						if ($distance->get()->asKilometres() <= $radius) {
							$place->Properties()->add($property);
							$place->save();
						}
					} catch (\Exception $e) {
						Injector::inst()->get(LoggerInterface::class)->info($e->getMessage());
					}
				}
			}
		}
	}

	public function providePermissions()
	{
		$permisions = [
			'MANAGE_PLACES' => [
				'name' => 'Manage places',
				'category' => 'Places',
			],
		];
		$this->extend('updatePermisions', $permisions);
		return $permisions;
	}

	public function canView($member = null)
	{
		return Permission::check(['MANAGE_PLACES']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_PLACES']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_PLACES']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_PLACES']);
	}
}
